require("Scripts/hud.lua")
require("Scripts/FlaskManager.lua")

--This function will be called once when the program starts
function App:Start()
	
	--Set the application title
	self.title="GreenFlask"
	
	--Create a window
	self.window=Window:Create(self.title)
	self.window:HideMouse()
	
	--Create the graphics context
	self.context=Context:Create(self.window,0)
	if self.context==nil then return false end
	
	--Create a world
	self.world=World:Create()
	
	--Load a map
	local mapfile = System:GetProperty("map","Maps/start.map")
	if Map:Load(mapfile)==false then return false end

	FlaskManager:Init()
	--_G.HUD = require("Scripts/hud.lua")

	self.restart = false
	
	return true
end

--This is our main program loop and will be called continuously until the program ends
function App:Loop()
	
	--If window has been closed, end the program
	if self.window:Closed() or self.window:KeyDown(Key.Escape) then return false end
	
	--Update the app timing
	Time:Update()
	
	--Update the world
	self.world:Update()
	
	--Render the world
	self.world:Render()

	self.context:SetBlendMode(Blend.Alpha)
	--self.context:DrawText("UPS: "..Time:UPS(),2,2)

	--local mouse = self.window:GetMousePosition()
	--print(Time:GetSpeed()*100)
	--self.context:DrawText((Time:GetSpeed()*100), 2, 16)

	_G.HUD:Draw()
	
	--Refresh the screen
	self.context:Sync()

	if self.restart == true then
		self:OnRestart()
	end

	-- Load Sounds
	_G.snd_flask = Sound:Load("Sounds/glassbreak.wav")
	_G.snd_explode = Sound:Load("Sounds/explode.wav")
	_G.snd_item = Sound:Load("Sounds/item.wav")
	_G.snd_keyitem = Sound:Load("Sounds/keyitem.wav")
	
	--Returning true tells the main program to keep looping
	return true
end

function App:Restart()
	self.restart = true
end


function App:OnRestart()
	self.world:Release()
	self.world=World:Create()
	--Load a map
	local mapfile = System:GetProperty("map","Maps/start.map")
	if Map:Load(mapfile)==false then return false end
	FlaskManager:Init()
	self.restart = false
end